@@ -8,21 +8,25 @@
 #define XGBOOST_METRIC_H_
 
 #include <dmlc/registry.h>
+#include <xgboost/generic_parameters.h>
+#include <xgboost/data.h>
+#include <xgboost/base.h>
+#include <xgboost/host_device_vector.h>
+
 #include <vector>
 #include <string>
 #include <functional>
 #include <utility>
 
-#include "./data.h"
-#include "./base.h"
-#include "../../src/common/host_device_vector.h"
-
 namespace xgboost {
 /*!
  * \brief interface of evaluation metric used to evaluate model performance.
  *  This has nothing to do with training, but merely act as evaluation purpose.
  */
 class Metric {
+ protected:
+  GenericParameter const* tparam_;
+
  public:
   /*!
    * \brief Configure the Metric with the specified parameters.
@@ -30,17 +34,6 @@ class Metric {
    */
   virtual void Configure(
       const std::vector<std::pair<std::string, std::string> >& args) {}
-  /*!
-   * \brief set configuration from pair iterators.
-   * \param begin The beginning iterator.
-   * \param end The end iterator.
-   * \tparam PairIter iterator<std::pair<std::string, std::string> >
-   */
-  template<typename PairIter>
-  inline void Configure(PairIter begin, PairIter end) {
-    std::vector<std::pair<std::string, std::string> > vec(begin, end);
-    this->Configure(vec);
-  }
   /*!
    * \brief evaluate a specific metric
    * \param preds prediction
@@ -63,7 +56,7 @@ class Metric {
    *  and the name will be matched in the registry.
    * \return the created metric.
    */
-  static Metric* Create(const std::string& name);
+  static Metric* Create(const std::string& name, GenericParameter const* tparam);
 };
 
 /*!
