@@ -11,21 +11,27 @@
 #include <xgboost/data.h>
 
 #include <algorithm>
-#include <cstring>
 #include <memory>
+#include <limits>
 #include <utility>
 #include <vector>
 
 #include "simple_csr_source.h"
+#include "../common/group_data.h"
+#include "../common/math.h"
+#include "adapter.h"
 
 namespace xgboost {
 namespace data {
-
+// Used for single batch data.
 class SimpleDMatrix : public DMatrix {
  public:
-  explicit SimpleDMatrix(std::unique_ptr<DataSource>&& source)
+  explicit SimpleDMatrix(std::unique_ptr<DataSource<SparsePage>>&& source)
       : source_(std::move(source)) {}
 
+  template <typename AdapterT>
+  explicit SimpleDMatrix(AdapterT* adapter, float missing, int nthread);
+
   MetaInfo& Info() override;
 
   const MetaInfo& Info() const override;
@@ -34,18 +40,18 @@ class SimpleDMatrix : public DMatrix {
 
   bool SingleColBlock() const override;
 
-  BatchSet GetRowBatches() override;
-
-  BatchSet GetColumnBatches() override;
-
-  BatchSet GetSortedColumnBatches() override;
-
  private:
-  // source data pointer.
-  std::unique_ptr<DataSource> source_;
+  BatchSet<SparsePage> GetRowBatches() override;
+  BatchSet<CSCPage> GetColumnBatches() override;
+  BatchSet<SortedCSCPage> GetSortedColumnBatches() override;
+  BatchSet<EllpackPage> GetEllpackBatches(const BatchParam& param) override;
 
-  std::unique_ptr<SparsePage> sorted_column_page_;
-  std::unique_ptr<SparsePage> column_page_;
+  // source data pointer.
+  std::unique_ptr<DataSource<SparsePage>> source_;
+
+  std::unique_ptr<CSCPage> column_page_;
+  std::unique_ptr<SortedCSCPage> sorted_column_page_;
+  std::unique_ptr<EllpackPage> ellpack_page_;
 };
 }  // namespace data
 }  // namespace xgboost
