@@ -3,8 +3,14 @@
  * \file gbm.cc
  * \brief Registry of gradient boosters.
  */
-#include <xgboost/gbm.h>
 #include <dmlc/registry.h>
+#include <string>
+#include <vector>
+#include <memory>
+
+#include "xgboost/gbm.h"
+#include "xgboost/learner.h"
+#include "xgboost/generic_parameters.h"
 
 namespace dmlc {
 DMLC_REGISTRY_ENABLE(::xgboost::GradientBoosterReg);
@@ -13,15 +19,17 @@ DMLC_REGISTRY_ENABLE(::xgboost::GradientBoosterReg);
 namespace xgboost {
 GradientBooster* GradientBooster::Create(
     const std::string& name,
-    const std::vector<std::shared_ptr<DMatrix> >& cache_mats,
-    bst_float base_margin) {
+    GenericParameter const* generic_param,
+    LearnerModelParam const* learner_model_param,
+    const std::vector<std::shared_ptr<DMatrix> >& cache_mats) {
   auto *e = ::dmlc::Registry< ::xgboost::GradientBoosterReg>::Get()->Find(name);
   if (e == nullptr) {
     LOG(FATAL) << "Unknown gbm type " << name;
   }
-  return (e->body)(cache_mats, base_margin);
+  auto p_bst =  (e->body)(cache_mats, learner_model_param);
+  p_bst->generic_param_ = generic_param;
+  return p_bst;
 }
-
 }  // namespace xgboost
 
 namespace xgboost {
