@@ -1,10 +1,16 @@
 /*!
- * Copyright by Contributors 2017
+ * Copyright 2017-2019 by Contributors
+ * \file gbtree_model.h
  */
-#pragma once
+#ifndef XGBOOST_GBM_GBTREE_MODEL_H_
+#define XGBOOST_GBM_GBTREE_MODEL_H_
+
 #include <dmlc/parameter.h>
 #include <dmlc/io.h>
+#include <xgboost/model.h>
 #include <xgboost/tree_model.h>
+#include <xgboost/parameter.h>
+#include <xgboost/learner.h>
 
 #include <memory>
 #include <utility>
@@ -12,47 +18,43 @@
 #include <vector>
 
 namespace xgboost {
+
+class Json;
+
 namespace gbm {
 /*! \brief model parameters */
 struct GBTreeModelParam : public dmlc::Parameter<GBTreeModelParam> {
+ public:
   /*! \brief number of trees */
-  int num_trees;
-  /*! \brief number of roots */
-  int num_roots;
+  int32_t num_trees;
+  /*! \brief (Deprecated) number of roots */
+  int32_t deprecated_num_roots;
   /*! \brief number of features to be used by trees */
-  int num_feature;
+  int32_t deprecated_num_feature;
   /*! \brief pad this space, for backward compatibility reason.*/
-  int pad_32bit;
+  int32_t pad_32bit;
   /*! \brief deprecated padding space. */
-  int64_t num_pbuffer_deprecated;
-  /*!
-   * \brief how many output group a single instance can produce
-   *  this affects the behavior of number of output we have:
-   *    suppose we have n instance and k group, output will be k * n
-   */
-  int num_output_group;
+  int64_t deprecated_num_pbuffer;
+  // deprecated. use learner_model_param_->num_output_group.
+  int32_t deprecated_num_output_group;
   /*! \brief size of leaf vector needed in tree */
-  int size_leaf_vector;
+  int32_t size_leaf_vector;
   /*! \brief reserved parameters */
-  int reserved[32];
+  int32_t reserved[32];
+
   /*! \brief constructor */
   GBTreeModelParam() {
-    std::memset(this, 0, sizeof(GBTreeModelParam));
-    static_assert(sizeof(GBTreeModelParam) == (4 + 2 + 2 + 32) * sizeof(int),
+    std::memset(this, 0, sizeof(GBTreeModelParam));  // FIXME(trivialfis): Why?
+    static_assert(sizeof(GBTreeModelParam) == (4 + 2 + 2 + 32) * sizeof(int32_t),
                   "64/32 bit compatibility issue");
+    deprecated_num_roots = 1;
   }
+
   // declare parameters, only declare those that need to be set.
   DMLC_DECLARE_PARAMETER(GBTreeModelParam) {
-    DMLC_DECLARE_FIELD(num_output_group)
-        .set_lower_bound(1)
-        .set_default(1)
-        .describe(
-            "Number of output groups to be predicted,"
-            " used for multi-class classification.");
-    DMLC_DECLARE_FIELD(num_roots).set_lower_bound(1).set_default(1).describe(
-        "Tree updater sequence.");
-    DMLC_DECLARE_FIELD(num_feature)
+    DMLC_DECLARE_FIELD(num_trees)
         .set_lower_bound(0)
+        .set_default(0)
         .describe("Number of features used for training and prediction.");
     DMLC_DECLARE_FIELD(size_leaf_vector)
         .set_lower_bound(0)
@@ -61,12 +63,14 @@ struct GBTreeModelParam : public dmlc::Parameter<GBTreeModelParam> {
   }
 };
 
-struct GBTreeModel {
-  explicit GBTreeModel(bst_float base_margin) : base_margin(base_margin) {}
-  void Configure(const std::vector<std::pair<std::string, std::string> >& cfg) {
+struct GBTreeModel : public Model {
+ public:
+  explicit GBTreeModel(LearnerModelParam const* learner_model_param) :
+      learner_model_param_{learner_model_param} {}
+  void Configure(const Args& cfg) {
     // initialize model parameters if not yet been initialized.
     if (trees.size() == 0) {
-      param.InitAllowUnknown(cfg);
+      param.UpdateAllowUnknown(cfg);
     }
   }
 
@@ -81,34 +85,11 @@ struct GBTreeModel {
     }
   }
 
-  void Load(dmlc::Stream* fi) {
-    CHECK_EQ(fi->Read(&param, sizeof(param)), sizeof(param))
-        << "GBTree: invalid model file";
-    trees.clear();
-    trees_to_update.clear();
-    for (int i = 0; i < param.num_trees; ++i) {
-      std::unique_ptr<RegTree> ptr(new RegTree());
-      ptr->Load(fi);
-      trees.push_back(std::move(ptr));
-    }
-    tree_info.resize(param.num_trees);
-    if (param.num_trees != 0) {
-      CHECK_EQ(
-          fi->Read(dmlc::BeginPtr(tree_info), sizeof(int) * param.num_trees),
-          sizeof(int) * param.num_trees);
-    }
-  }
+  void Load(dmlc::Stream* fi);
+  void Save(dmlc::Stream* fo) const;
 
-  void Save(dmlc::Stream* fo) const {
-    CHECK_EQ(param.num_trees, static_cast<int>(trees.size()));
-    fo->Write(&param, sizeof(param));
-    for (const auto & tree : trees) {
-      tree->Save(fo);
-    }
-    if (tree_info.size() != 0) {
-      fo->Write(dmlc::BeginPtr(tree_info), sizeof(int) * tree_info.size());
-    }
-  }
+  void SaveModel(Json* p_out) const override;
+  void LoadModel(Json const& p_out) override;
 
   std::vector<std::string> DumpModel(const FeatureMap& fmap, bool with_stats,
                                      std::string format) const {
@@ -128,7 +109,7 @@ struct GBTreeModel {
   }
 
   // base margin
-  bst_float base_margin;
+  LearnerModelParam const* learner_model_param_;
   // model parameter
   GBTreeModelParam param;
   /*! \brief vector of trees stored in the model */
@@ -140,3 +121,5 @@ struct GBTreeModel {
 };
 }  // namespace gbm
 }  // namespace xgboost
+
+#endif  // XGBOOST_GBM_GBTREE_MODEL_H_
