@@ -22,6 +22,9 @@ DMLC_REGISTRY_FILE_TAG(updater_skmaker);
 
 class SketchMaker: public BaseMaker {
  public:
+  char const* Name() const override {
+    return "grow_skmaker";
+  }
   void Update(HostDeviceVector<GradientPair> *gpair,
               DMatrix *p_fmat,
               const std::vector<RegTree*> &trees) override {
@@ -132,7 +135,7 @@ class SketchMaker: public BaseMaker {
     // number of rows in
     const size_t nrows = p_fmat->Info().num_row_;
     // start accumulating statistics
-    for (const auto &batch : p_fmat->GetSortedColumnBatches()) {
+    for (const auto &batch : p_fmat->GetBatches<SortedCSCPage>()) {
       // start enumeration
       const auto nsize = static_cast<bst_omp_uint>(batch.Size());
       #pragma omp parallel for schedule(dynamic, 1)
